/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_isr.c
*
* @brief This file provides BLDC control interruptcallback functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_pwdt.h"
#include "ac78xx_adc.h"
#include "bldc_keys.h"
#include "bldc_isr.h"
#include "bldc_pwm.h"
#include "bldc_adc.h"
#include "bldc_bemf.h"
#include "task_scheduler.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Callback routine of TIMER1, 1ms. The Task scheduler update in Timer1 interrupt,use for bldc control time base. 
*
* @param[in] argc: callback parameter
* @return none
*/
void BLDC_Base_TimerCallback(uint8_t argc)
{
    /* same as FOC_TIMER callback, to update the time base tasks in TIMER_ISR */
    if (TIMER_GetIntFlag(TIMER1))
    {
        SCH_Update();
        TIMER_ClrIntFlag(TIMER1);
    }
}

/*!
* @brief Callback routine of TIMER2, use for back EMF zero crossing detection in bldc sensorless control. 
*
* @param[in] argc: callback parameter
* @return none
*/
void Timer2_Callback(uint8_t argc)
{
    if (TIMER_GetIntFlag(TIMER2))
    {
        TIMER_ClrIntFlag(TIMER2);
        if (Get_McStatus() == RUN)
        {
            /* 60000 * 6 * TIMER_XUS = 8.64s, constrained the minmum operating elec frequency 0.12Hz */
            if (g_bemfControl.changePhaseTime < 60000)
            {
                g_bemfControl.changePhaseTime++;
            }
            BemfCheckFun(&g_bldcVarsCtrl, pBldcVarsCfg);
        }
    }
}

/*!
* @brief Callback routine of ADC, use for ADC sample data and current loop in bldc control.
*
* @param[in] none
* @return none
*/
void BLDC_ADCCallback(void)
{
    if (ADC_GetIntFlag(ADC, ADC_FLAG_EOC))
    {
        ADC_ClearFlag(ADC, ADC_FLAG_EOC);
    }
    if (ADC_GetIntFlag(ADC, ADC_FLAG_AMO))
    {
        ADC_ClearFlag(ADC, ADC_FLAG_AMO);
    }
    if (ADC_GetIntFlag(ADC, ADC_FLAG_IEOC))
    {
        ADC_ClearFlag(ADC, ADC_FLAG_IEOC);
        #if (defined DAC_FUNCTIONALITY)
        PWM_SetChannelValue(PWM0, PWM_CHANNEL_CHANNEL0, ((g_bldc_speedCmd.speedTargetRamp + 32768) >> 4));
        PWM_SetChannelValue(PWM0, PWM_CHANNEL_CHANNEL1, ((g_bldc_speedCmd.speedFbk + 32768) >> 4));
        #endif

        #if (defined BEMF_DETECT_SOFTWARE)
        Get_PhaseVoltageValue(&g_bldc_adSample);
        #endif

        if ((Get_McStatus() == FAULT) || (Get_McStatus() == STOP) || (Get_McStatus() == IDLE) || (Get_McStatus() == POWER_ON))
        {
            BLDC_Stop();
            return;
        }
        else if (Get_McStatus() == RUN)
        {
            BusCurrent_Control(&g_bldcVarsCtrl, &g_bldc_adSample);
        }
        else
        {

        }
    }
}

/*!
* @brief Callback routine of PWM, occur while hardware over current trigger PWM fault, use for hardware over current protection in bldc control.
*
* @param[in] none
* @return none
*/
void BLDC_PWM_FaultCallback(void)
{
    if (PWM_GetOverFlowFlag(PWM2))
    {
        PWM_ClrOverFlowFlag(PWM2);
    }
    if (PWM_GetFaultDetectionFlag(PWM2, (uint16_t)PWM_EXTERNAL_FAULT1))
    {
        Set_McStatus(FAULT);
        g_bldc_protector.faultFlag.bits.hwFault = 1;
        PWM_ClrFaultDetectionFlag(PWM2, (uint16_t)PWM_EXTERNAL_FAULT1);
    }
}

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for back EMF zero crossing detection hall signal captrue in bldc sensorless control.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_PWDTCallback(void)
{

}

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for hall signal captrue in bldc control with hall sensor.
*
* @param[in] none
* @return none
*/
void BLDC_Hall_PWDTCallback(void)
{
    uint32_t temp_PwdtNpw;
    uint32_t temp_PwdtInit0;
    temp_PwdtNpw = PWDT->NPWCV;
    temp_PwdtInit0 = PWDT->INIT0;

    if (PWDT_GetOverflowFlag())
    {
        PWDT_ClrOverflowFlag();
        memset(&g_hallControl.totalPulseWidth, 0U , sizeof(g_hallControl.totalPulseWidth));
        if (0 == g_bldc_protector.stallFlag)
        {
            g_hallControl.pwdtValid = 0;
        }
        else
        {
            g_hallControl.currentPhase = PWDT_GetHallStatus() & 0x07;
            if (Get_McStatus() == RUN)
            {
                BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
            }
        }
        return;
    }
    if (PWDT_GetReadyFlag())
    {
        temp_PwdtNpw = PWDT->NPWCV;
        temp_PwdtInit0 = PWDT->INIT0;
        PWDT_ClrReadyFlag();
        if (g_hallControl.pwdtValid < 2)
        {
            g_hallControl.pwdtValid++;
            return;
        }
        g_hallControl.currentPhase = PWDT_GetHallStatus() & 0x07;
        StorageDataCyclic(&g_hallControl.totalPulseWidth, ((temp_PwdtNpw & 0xFFFF) + ((temp_PwdtInit0 >> 16) & 0xFFFF)));
        if (Get_McStatus() == RUN)
        {
            BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
        }
    }
}

/* =============================================  EOF  ============================================== */
