/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_isr.c
*
* @brief This file provides FOC control interruptcallback functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_gpio.h"
#include "ac78xx_pwm.h"
#include "ac78xx_adc.h"
#include "ac78xx_timer.h"
#include "foc_isr.h"
#include "foc_keys.h"
#include "motor_debug.h"
#include "globalvar.h"
#include "vf_control.h"
#include "task_scheduler.h"
#include "pwmdrv.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief PWMx Fault Test CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] none
* @return none
*/
void FOC_PWM_FaultCallback(void)
{
    if (PWM_GetOverFlowFlag(PWM2))
    {
        PWM_ClrOverFlowFlag(PWM2);
    }
    if (PWM_GetFaultDetectionFlag(PWM2, PWM_EXTERNAL_FAULT2))
    {
        Set_McStatus(FAULT);
        PWM_ClrFaultDetectionFlag(PWM2, PWM_EXTERNAL_FAULT2);
        PWM_Output(0);
        g_protector.ErrFlag.bits.hwFault = 1;
        DSP2[0] = LEDBL;
        DSP2[1] = LED_E;
        DSP2[2] = LEDBAR;
        DSP2[3] = LED0;
        DSP2[4] = LED2;
    }
}

/*!
* @brief Callback routine of TIMER1. The interruption period is 1ms. The Task scheduler update in Timer interrupt.
*        When the Timer_1 counts to 1ms, the Timer_1 interrupt is triggered and this function is called.
*
* @param[in] argc: callback parameter
* @return none
*/
void Timer1_Callback(uint8_t argc)
{
    if (TIMER_GetIntFlag(TIMER1))
    {
        SCH_Update();
        TIMER_ClrIntFlag(TIMER1);
    }
}

/*!
* @brief Callback routine of TIMER2. The interruption period is M_T_PERIOD. The Task scheduler update in Timer interrupt.
*        When the Timer_2 counts to M_T_PERIOD, the Timer_2 interrupt is triggered and this function is called.
*
* @param[in] argc: callback parameter
* @return none
*/
void Timer2_Callback(uint8_t argc)
{
    if (TIMER_GetIntFlag(TIMER2))
    {
        /* query the rising edge of pulse and counting */
        #if (defined M_T_SPEED_MEASURE)
        Encoder_M_T_Measure(&g_encoderSpeed);
        #endif
        TIMER_ClrIntFlag(TIMER2);
    }
}

/*!
* @brief Call back function of PWDT modular. The PWDT modular is mainly used by hall sensor.
*        When PWDT modular input signal level change or PWDT timer overflow will trriger PWDT interrupt.
*
* @param[in] none
* @return none
*/
void FOC_PWDT_Callback(void)
{
    uint8_t hallStatus = 0;
    uint32_t posWidth = 0;
    uint32_t negWidth = 0;

    if (PWDT_GetOverflowFlag())
    {
        PWDT_ClrOverflowFlag();
        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
            HallSensor_Init(PWDT_GetHallStatus(), &g_hall, &g_focVars);
        }

//        return;
    }
    if (PWDT_GetReadyFlag())
    {
        /* get pwdt1 HALL status */
        hallStatus = PWDT_GetHallStatus();
        PWDT_ClrReadyFlag();

        negWidth = (PWDT->NPWCV) & 0xffff;
        posWidth = (PWDT->INIT0 >> 16) & 0xffff;

        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
            HallSensor_PwdtSubCallBack(hallStatus, negWidth, posWidth, &g_hall, pFocVarsCfg);
            #if (defined ENCODER_HALL_SENSORS)
            EncoderHall_AngleAdjust(&g_encoderHallSensor, &g_hallSensor);
            #endif

            if (g_hall.pHallSensor->pwdtDatValid == 2U)
            {
                if (Hall_Sensor_Change_Check(hallStatus, g_hall.pHallSensor->hallStatus, &g_protector))
                {
                    HallSensor_Init(hallStatus, &g_hall, &g_focVars);
                    Set_McStatus(FAULT);
                }
            }
        }
        else
        {
            HallSensor_SelfLearnPwdt(hallStatus, negWidth, posWidth, &g_hall);
        }

    }
#if (defined HALL_SENSORS)
    if ((Get_McStatus() != STOP) && (Get_McStatus() != IDENTIFY))
    {
        //Hall_Sensor_Status_Check(g_hallSensor.hallStatus);
        //Hall_Sensor_Change_Check(g_hallSensor.hallStatus,hallStatusOld);
        //hallStatusOld = g_hallSensor.hallStatus;
//        HallFlyingCarProtect();
    }
#endif

}

/*!
* @brief PWM1 count overflow interrupt to statistic circles for encoder position servo applicaton.
*
* @param[in] none
* @return none
*/
void FOC_Encoder_Callback(void)
{

    if (PWM_GetOverFlowFlag(PWM1))
    {
        PWM_ClrOverFlowFlag(PWM1);
        #if (defined POSITION_LOOP_CTRL)
        Encoder_MotorCircleCalc(&g_encoderSensor);
        #endif
    }
}

/*!
* @brief Encoder Z single gpio interrupt to fix error for ABZ encoder applicaton.
*
* @param[in] none
* @return none
*/
void GPIO_EncoderZ_callback(void)
{
    if (g_encoderSensor.corssZFlag == 0)
    {
        if (g_startUp.pStCtrl->alignState == 1)
        {
            g_encoderSensor.corssZFlag = 1;
            Encoder_GetPulseCount(&g_encoderSensor);
            g_encoderSensor.zCntOffSetBase = g_encoderSensor.encoderCnt;
            g_encoderSensor.zCntOffSet = g_encoderSensor.zCntOffSetBase;
            g_encoderSensor.encoderCntOffSet = g_encoderSensor.zCntOffSet;
        }
        else
        {
            return;
        }
    }
    else
    {
    }
}

/*!
* @brief ADC interrupt service subfunction for short brake.
*        When motor is short circuit braking, ADC interruption wiil enter this function.
*
* @param[in] none
* @return none
*/
void ShortBrake_AdcProcess(void)
{
    Get_CurrentSample(&g_adcSampleReal, &g_adcSampleCof, &g_adcTransform);
    AdcResultTransmit(&g_adcTransform, &g_focVarsCtrl);
    ShortBrake_AdcSubCallBack(&g_shortBrake, &g_focVars, pShortBrakeCfg, &g_motorParamPu);
}

/*!
* @brief Hall sensor self-learn process.
*
* @param[in] none
* @return none
*/
void HallSelfLearnProcess(void)
{
    if (defHallSensor.hallSelfLearnMode == 1)
    {
        if (g_hallSelfLearn.selfLearnStage != SELF_LEARN_2DEGREE_FINISHED)
        {
            HallSensor_SelfLearn(PWDT_GetHallStatus(), &g_hall, &g_focVars);
            g_hallSelfLearnFinishFlag = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag = 1;
        }
    }
    else if (defHallSensor.hallSelfLearnMode == 2)
    {
        if (g_hallSelfLearn.selfLearnStage != SELF_LEARN_HALL_WIDTH)
        {
            HallSensor_SelfLearn(PWDT_GetHallStatus(), &g_hall, &g_focVars);
            g_hallSelfLearnFinishFlag = 0;
        }
        else
        {
            g_hallSelfLearnFinishFlag = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief The first part of the FOC interrupt to obtain the electrical angle.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Foc_Execution_AngleProcess(FOC_VARS *focVars)
{
    if (Get_McStatus() != IDENTIFY)
    {
#if (defined FOC_PMSM)
    #if (defined ENCODER_HALL_SENSORS)
        if (defHallSensor.hallSelfLearnEnable == ENABLE)
        {
            HallSelfLearnProcess();
        }
        else if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
        {
            EncoderHall_AngleCal(&g_encoderHallSensor);
            #if (defined POSITION_LOOP_CTRL)
            Encoder_ElecAngleCalc(&g_encoderSensor, pMotorParam);
            #endif
        }
        else
        {
        }
    #elif (defined HALL_SENSORS)
        if (Get_McStatus() == HALL_SELFLEARN)
        {
            HallSelfLearnProcess();
        }
        else if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
        {
            HallSensor_AdcCallBack(PWDT_GetHallStatus(), &g_hall);
        }
        else
        {
        }
    #elif (defined ENCODER_SENSORS)
        Encoder_ElecAngleCalc(&g_encoderSensor, pMotorParam);
        #if ((!defined M_T_SPEED_MEASURE) && (defined ELEC_FREQUENCE_HIGH))
        Asr_EstSpeedCalc(g_encoderSensor.encoderElecAngle, &g_rotationalSpeed);
        #endif

    #elif (defined SENSORLESS)
        if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
        {
            #if (defined HIGH_FREQUENCE_INJECTION)
            #elif (defined IF_STARTUP) || (defined DIRECT_STARTUP) || (defined VF_STARTUP)
            StartUp_Ctrl_Foc(focVars, &g_startUp);
            #endif
        }
    #else
    #endif

#elif (defined FOC_ACIM)
    #if (defined SENSOR_ACIM)
        Acim_SensorSpeedCalc(&g_acimFocCtrl, &g_STC, &g_encoderSensor, &g_rotationalSpeed);
    #elif (defined SENSORLESS_ACIM)
        if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN) || (Get_McStatus() == ALIGN))
        {
            StartUp_Ctrl_Foc(focVars, &g_startUp);
        }
    #else
    #endif
#endif
    }
    else
    {
#if (defined FOC_PMSM)
        if ((PMSM_LDQ_METHOD_HFS == g_paramIdentify.pParamIdentifyCfg->pmsmLdqTypeSel) \
            && (IDENTIFY_LDQ_STALLTEST == g_paramIdentify.pParamIdentifyCtrl->mainStep))
        {
//            PmsmId_LdqCalcIsr(&g_paramIdentify, &g_STC, &g_sigPhaseShift);

            PMSMLdqId_IsrTask(&g_paramIdentify, &g_STC);
        }
#elif (defined FOC_ACIM)
        if (IDENTIFY_RS_DCTEST != g_paramIdentify.pParamIdentifyCtrl->mainStep)
        {
            AcimId_IdentificationIsr(&g_paramIdentify, &g_sigPhaseShift, &g_STC);
        }
#else
#endif
    }

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN) || (Get_McStatus() == IDENTIFY) || (Get_McStatus() == HALL_SELFLEARN))
    {
        Get_ElecAngle(&g_focVarsCtrl);
    }

    #if (defined TAIL_HEAD_WIND)
    if ((Get_McStatus() == SPEED_JUDGE) || (Get_McStatus() == FLY_RUN))
    {
        Speed_Judge_Process(&g_initSpeedJudge, &g_focVars);
        if (g_speedCommand.initStatus == HIGH_FORWARD)
        {
            g_pidPll.pPidOutput->uiPu = g_initSpeedJudge.initSpeed;
//            focVars->pCtrl->elecAngle = g_initSpeedJudge.bemfTheta;
            #if (defined FLUX_OBSERVE)
            g_fluxObserverCtrl.estTheta = g_initSpeedJudge.bemfTheta;
            #elif (defined SMC_OBSERVE)
            g_smcEstimator.estTheta = g_initSpeedJudge.bemfTheta;
            #elif (defined MRAS_OBSERVE)
            g_mrasObserver.estFreqBuf = g_initSpeedJudge.initSpeed;
            g_mrasObserver.estTheta = g_initSpeedJudge.bemfTheta;
            #endif
            ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, g_speedCommand.initSpeedJudgeSpeed);
            ASR_SetSpeedTarget_Pu(&g_speedCommand, g_speedCommand.initSpeedJudgeSpeed);
            Set_McStatus(RUN);
            StartUp_Motor();
            g_startUpCtrl.startUpFlag = 4;
        }
    }
    else
    {
    }
    #endif
}

/*!
* @brief The second part of the FOC interrupt to execute current loop calculation and generate SVPWM modulation wave.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Foc_Execution_CurrentLoopProcess(FOC_VARS_CTRL *focVarsCtrl)
{
#if (defined FOC_SINE_WAVE)
    #if (defined FOC_PMSM)
    #if (defined SENSORLESS) && (defined VOLTAGE_RESTRUCT)
    Smc_GetThreePhaseVolt(focVarsCtrl, &g_pwmValue);
    #endif
    #elif (defined FOC_ACIM)
    Smc_GetThreePhaseVolt(focVarsCtrl, &g_pwmValue);
    #endif
    CurrentLoop_Process(focVarsCtrl);
#elif (defined VF_CONTROL)
    Vf_ISR();
#else
#endif
}

/*!
* @brief ADC interrupt service for FOC control.
*
* @param[in] none
* @return none
*/
void Motor_Ctrl_CallBack(void)
{
    if (ADC_GetIntFlag(ADC, ADC_FLAG_EOC))
    {
        ADC_ClearFlag(ADC, ADC_FLAG_EOC);
    }
    if (ADC_GetIntFlag(ADC, ADC_FLAG_AMO))
    {
        ADC_ClearFlag(ADC, ADC_FLAG_AMO);
    }
    if (ADC_GetIntFlag(ADC, ADC_FLAG_IEOC))
    {
        #ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
        GPIO_SetPinValue(DEBUG_GPIO, 1);
        #endif
        ADC_ClearFlag(ADC, ADC_FLAG_IEOC);
        if ((Get_McStatus() == FAULT) || (Get_McStatus() == STOP) || (Get_McStatus() == IDLE) || (Get_McStatus() == POWER_ON))
        {
            #if (defined SENSOR_ACIM)
            Encoder_GetPulseCount(&g_encoderSensor);
            #endif
            #if ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
            Encoder_GetPulseCount(&g_encoderSensor);
            #endif
            PWM_Disable();
            return;
        }
        else if (Get_McStatus() == IDENTIFY)
        {
            if (((g_STC.pMotor->motorType == MOTOR_SPMSM) || (g_STC.pMotor->motorType == MOTOR_IPMSM)) \
                && ((g_paramIdentify.pParamIdentifyCtrl->mainStep != (uint8_t)IDENTIFY_RS_DCTEST) \
                && (ParamIdentify_AdcSubCallBack(&g_focVars))))
            {
                /* PMSM: If Rs identification is in progress, use the FOC ISR; otherwise, use param-identify ISR */
                return;
            }
            if (g_paramIdentify.pParamIdentifyCtrl->mainStep == (uint8_t)IDENTIFY_END)
            {
                return;
            }
        }
        else if ((Get_McStatus() == START_BRAKE) || (Get_McStatus() == STOP_BRAKE))
        {
            ShortBrake_AdcProcess();
            return;
        }
        else if (Get_McStatus() == ANGLE_JUDGE)
        {
            InitPos_AdcSubCallBack(&g_pulseInject, pPulseInjCfg);
            return;
        }
        else
        {
        }

        #if (FOC_EXECUTION_RATE == 1)
        Get_CurrentSample(&g_adcSampleReal, &g_adcSampleCof, &g_adcTransform);
        AdcResultTransmit(&g_adcTransform, &g_focVarsCtrl);
        Foc_Execution_AngleProcess(&g_focVars);
        Foc_Execution_CurrentLoopProcess(&g_focVarsCtrl);

        #elif (FOC_EXECUTION_RATE == 2)
        if (g_focVarsCtrl.focExecutionCycleCnt < (FOC_EXECUTION_RATE - 1))
        {
            g_focVarsCtrl.focExecutionCycleCnt ++;
            Foc_Execution_AngleProcess(&g_focVars);
        }
        else
        {
            g_focVarsCtrl.focExecutionCycleCnt = 0;
            Get_CurrentSample(&g_adcSampleReal, &g_adcSampleCof, &g_adcTransform);
            AdcResultTransmit(&g_adcTransform, &g_focVarsCtrl);
            Foc_Execution_CurrentLoopProcess(&g_focVarsCtrl);
        }
        #else
        #endif

        #if (defined DAC_FUNCTIONALITY)
        Debug_Display_Variables();
        #endif

        #ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
        GPIO_SetPinValue(DEBUG_GPIO, 0);
        #endif
    }
}

/* =============================================  EOF  ============================================== */
