/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file pulse_inject_api.c
*
* @brief This file mainly provides mcu PWM and ADC modular operation function aviod read or wirte mcu registers in libray.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_adc.h"
#include "ac78xx_ctu.h"
#include "pulse_inject_api.h"
#include "motorctrl_api.h"
#include "foc_isr.h"
#include "globalvar.h"
#include "pwmdrv.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Initial position pulse injection function main status machine, which include five status.
*        CHIP_CONFIG: pulse injection pwm and adc configuration.
*        SELF_LEARNING:  get proper pulse injection width, which is different with each motor.
*        PULSE_INJECTING: pulse injection process and result calculate.
*        CHIP_CONFIG_RESET: pulse injection finished, recover pwm and adc configuration.
*        PULSE_INJECTION_ERR: during pulse injection process, maybe occur some errors.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return Initial position pulse inject function finished flag
*/
uint16_t PulseInject(PULSE_INJECT_DATA *pulseInject)
{
//    uint16_t errCntTemp = 0;
    switch (pulseInject->subStatus)
    {
    case CHIP_CONFIG:
    PulseInject_Config(pulseInject);
    break;

    case SELF_LEARNING:
    PulseInject_SelfLearn(pulseInject);
    break;

    case PULSE_INJECTING:
    PulseInject_DataHandle(pulseInject);
    break;

    case CHIP_CONFIG_RESET:
    PulseInject_ConfigReset(pulseInject);
    break;

    case PULSE_INJECTION_ERR:
    pulseInject->pulseInjErrCnt = 1;
    pulseInject->subStatus = (uint8_t)CHIP_CONFIG_RESET;

    break;

    default:
    break;
    }
    return (pulseInject->pulseInjDoneFlag);
}

/*!
* @brief Beacuse the motor manbe bldc or pmsm, the control mode is different.
*        In order to output pulse injection, PWM and ADC module will be configurated, and then go to the next status.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Config(PULSE_INJECT_DATA *pulseInject)
{
    PWM_Disable();

    #ifndef PULSE_SELF_LEARN
    pulseInject->selfLearnFlag = 1;
    pulseInject->pwmTs = PWM_SELF_LEARN_PRD;
    #else
    pulseInject->pwmTs = INIT_PULSE_PERIOD;
    #endif

    CTU_TriggerAdcInjectGroupByPwm2Init();
    CTU_SetBusDivide(0);

    CTU_SetPwm2Trig2AdcDelayCnt(0);

    /* IBUS */
//    ADC_SetIntervalMode(ADC, DISABLE);
    ADC_SetInjectGroupLength(ADC,1);
    ADC_SetInjectGroupSequence(ADC, 1, CURRENT_IBUS);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IBUS, ADC_SampleTime_6Cycle);

    PWM_SetModValue(PWM2, (pulseInject->pwmTs - 1));

    /* set PWM2 Init Trigger Source */
    PWM_SetExternalTrigger(PWM2, PWM_EXTTRIG_INITTRIGEN_Msk);

    PulseInj_SetChlVal(7);
    PWM_SoftwareSync(PWM2);
    PWM_Enable();
//    PWM_SoftwareSync(PWM2);

    pulseInject->subStatus = (uint8_t)SELF_LEARNING;
}

/*!
* @brief Initial position pulse injection function finished, then recover configuration.
*        Because enter initial position module, bldc or pmsm control have different configuration, after initial position checked,
*        recover configuration according to control method.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ConfigReset(PULSE_INJECT_DATA *pulseInject)
{
    NVIC_DisableIRQ(ADC_IRQn);
    PWM_Disable();
    PulseInj_SetChlVal(6);

    PWM_ModuleDisable(PWM2);
    CTU_ModuleDisable();
    ADC_ModuleDisable(ADC);

    FOC_CTU_Initialize();
    FOC_ADC_Initialize();

    /* must behind adc init */
    FOC_PWM_Initialize();

    NVIC_EnableIRQ(ADC_IRQn);
    pulseInject->pulseInjDoneFlag = 1;
}

/*!
* @brief Initial position checking need inject at different direction, so this function can change pwm output.
*        This function can fit to different hardboard, through change settings.
*        What's more, PWM configuration makes pwm output mask effective immediately.
*
* @param[in] section: Judge pulse inject direction
* @return none
*/
void PulseInj_SetChlVal(uint8_t section)
{ /*!< PWM2_SYNCONF->SWOC = 0,CHOSWCR control reload immediately */
    switch (section)
    {
    /* A+:0, B-:3, C-:5 */
    case 0:

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_INACTIVE_STATE);
    break;

    /* A-:1, B+:2, C+:4 */
    case 1:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_INACTIVE_STATE);
    break;

    /* A-:1, B+:2, C-:5 */
    case 2:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_INACTIVE_STATE);
    break;

    /* A+:0, B-:3, C+:4 */
    case 3:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_INACTIVE_STATE);
    break;

    /* A-:1, B-:3, C+:4 */
    case 4:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_INACTIVE_STATE);
    break;

    /* A+:0, B+:2, C-:5 */
    case 5:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_ACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_ACTIVE_STATE);

    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_INACTIVE_STATE);
    break;

    /* recover */
    case 6:
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL0);
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL1);
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL2);
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL3);
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL4);
    PWM_ClrOutputSWControl(PWM2, PWM_CHANNEL_CHANNEL5);
    break;

    case 7:
    default:
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL0, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL1, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL2, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL3, DOWN_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL4, UP_BRIDGE_INACTIVE_STATE);
    PWM_OutputSWControlSet(PWM2, PWM_CHANNEL_CHANNEL5, DOWN_BRIDGE_INACTIVE_STATE);
    break;
    }
}

/*!
* @brief In order to make libray decouple with mcu registers, this function is called by libray to get bus current value.
*        It is called when pulse injection finished, and sample bus current.
*
* @param[in] none
* @return bus current AD value
*/
uint16_t PulseInject_GetBusCur(void)
{
    return (ADC_GetInjectedConversionValue(ADC, ADC_InjectedChannel_1));
}

/*!
* @brief Pulse injection function data clear.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Init(PULSE_INJECT_DATA *pulseInject)
{
    memset(pulseInject, 0, sizeof(*pulseInject));
}

/*!
* @brief Pulse injection function data clear.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ReJdg_Init(PULSE_INJECT_DATA *pulseInject)
{
    pulseInject->pulseInjDoneFlag = 0;
    pulseInject->periodCnt = 0;
    pulseInject->pulseInjFlag = 0;
    pulseInject->selfLearnFlag = 0;
    pulseInject->subStatus = 0;
    pulseInject->wait = 0;
}

/*!
* @brief Multiple pulse injection process in main status machine(ANGLE_JUDGE).
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return Multiple pulse inject function flag
*         0: pulse injection repeat unfinished, 1: pulse injection repeat finished, 2: pulse injection repeat results are inconsistent
*/
uint16_t PulseInject_Process(PULSE_INJECT_DATA *pulseInject, STARTUP_CTRL *startUp)
{
    uint16_t index = 0;
    uint16_t pulseInjRepeatDoneFlag = INIT_UNKNOWN;
    static uint16_t pulseInjectSection[PULSE_INJECTION_TIMES] = {0};
    static uint16_t pulseInjectCnt = 0;

    if (pulseInjectCnt < PULSE_INJECTION_TIMES)
    {
        pulseInjRepeatDoneFlag = INIT_UNKNOWN;
        if (PulseInject(pulseInject))
        {
            if (pulseInject->pulseInjErrCnt == 1)
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = EXE_FAIL;
                PulseInject_ReJdg_Init(pulseInject);
                return (pulseInjRepeatDoneFlag);
            }
            pulseInjectSection[pulseInjectCnt] = pulseInject->flowState;
            if ((pulseInjectCnt >= 1) && (pulseInjectSection[pulseInjectCnt] != pulseInjectSection[pulseInjectCnt - 1]))
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = EXE_FAIL;
                PulseInject_ReJdg_Init(pulseInject);
                /* init speed buffer */
                for (index = 0u; index < PULSE_INJECTION_TIMES; index++)
                {
                    pulseInjectSection[index] = 0;
                }
                return (pulseInjRepeatDoneFlag);
            }
            else
            {
            }
            pulseInjectCnt++;

            startUp->startUpElecTheta = InitPos_GetPosAngle(&g_pulseInject);

            PulseInject_ReJdg_Init(pulseInject);
        }
    }
    else
    {
        pulseInjectCnt = 0;
        pulseInjRepeatDoneFlag = EXE_SUCCESS;
        /* init speed buffer */
        for (index = 0u; index < PULSE_INJECTION_TIMES; index++)
        {
            pulseInjectSection[index] = 0;
        }
    }

    return (pulseInjRepeatDoneFlag);
}

/* =============================================  EOF  ============================================== */
